% Songyot Nakariyakul 
% Last modified: 13 August 2005

%SigOrder.m orders the feature set by significance (used in Step 1 of our ABB algorithm)

function [ftrIndex, m, Jall] = SigOrder(Input, groundtruth, J0);

% reference: S. Nakariyakul and D. Casasent, Adaptive branch and bound algorithm for selecting
%   optimal features, Pattern Recognition Letters, vol. 28(12), pp.1415-1427, 2007.

% 
% ftrIndex - the index of ordered features; the most significant feature is the first element, etc	
% Jall = the J value using ALL features
% m is the starting search level k_min

% Input is an numSxn input matrix, where numS is the number of training set samples and 
%     n is the total number of features.  These are the training set data.
% groundtruth - the class indicator for Input: 1 or not 1.
% J0 - The J value obtained from IFFS, used to calculate zmin (the starting search level)

numJ = 0;  % number of J calculations
[n] = size(Input,2);   % n = the total number of features
ftrIndex = 1:n;  % intialize ftrIndex; it can be any dummy number

Jall = computeJ(Input, groundtruth);  %Jall = the J value of the whole feature set

% Initialization
n = size(Input, 2);
ftrIndex = [];
ftrs = 1:n;
m = [];
numM = length(ftrs);  % numM is the number of features that have not been ordered

% this While loop orders the feature set by significance
while numM > 1
    
    J = [];
    
    for i = 1:numM
        ftmp = ftrs; ftmp(i) = [];
        J(i) = computeJ(Input(:,ftmp), groundtruth); numJ = numJ+1;
    end
   
    
    Index = find(J==min(J));
    ftrIndex = [ftrIndex ftrs(Index)];  %ftrIndex updates the list of the most sig features
    ftrs(Index) = [];
    
    numM = length(ftrs);   %update the number of features that have not been ordered
    
    if min(J) < J0; m = [m length(ftrIndex)]; end  %find levels that 
    
end

m = min(m);  %find the starting search level
ftrIndex = [ftrIndex ftrs];

