%Songyot Nakariyakul
%12/01/05
%computeJ
%computeJ is used to compute the J value for the input matrix.  
%Input is an SxF data matrix, where S is the number of pixel training set samples and F is the
%number of features.
%     In our work, training set samples are extracted from pixels in HS images; each row in Input is
%     the response at one pixel.
%groundtruth is a Sx1 ground truth vector for the S samples in Input.   
%    We only consider the 2-class problem. Entries in groundtruth are 1 for class 1 and not 1 for the other class  
%    There are 4 distance criterion functions one can choose (Bhattacharyya, Mahalanobis, Patrick-Fischer, Divergence)
%    The user uncomments the measure he wants to use. 
function J = computeJ(Input, groundtruth);   

[S, F] = size(Input);   % S -- samples,  F -- features
c1 = find(groundtruth==groundtruth(1));  % class 1 samples
c2 = find(groundtruth~=groundtruth(1));  % class 2 samples

Class1 = Input(c1, :);
Class2 = Input(c2, :);

m1 = mean(Class1);      %the mean vector of class 1 samples
m2 = mean(Class2);      %the mean vector of class 2 samples  
Cov1 = cov(Class1);     %the covariance matrix of class 1 samples    
Cov2 = cov(Class2);     %the covariance matrix of class 2 samples


%%%%%%%%%%%%%%%%%% Bhattacharyya Distance %%%%%%%%%%%%%%%%%%%%%


%J = (m1-m2)*inv((Cov1+Cov2)/2)*(m1-m2)'/8 +log(det((Cov1+Cov2)/2)/sqrt(det(Cov1)*det(Cov2)))/2;


%%%%%%%%%%%%%%%%% Mahalanobis Distance %%%%%%%%%%%%%%%%%%%%%%%%

covm = (size(Class1,1)/S)*Cov1+(size(Class2,1)/S)*Cov2;
J = (m1-m2)*inv(covm)*(m1-m2)';


%%%%%%%%%%%%%%%%% Patrick-Fischer Distance (see FBB) %%%%%%%%%%%

%J = (2*pi)^(F/2)*(det(2*Cov1)^-0.5+det(2*Cov2)^-0.5 - 2*det(Cov1+Cov2)^-0.5 ...
%    *exp((m2-m1)*inv(Cov1+Cov2)*0.5*(m2-m1)'));


%%%%%%%%%%%%%%%%% Divergence %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
%invC1 = inv(Cov1);  invC2 = inv(Cov2);
%J = (m2-m1)*(invC1+invC2)*0.5*(m2-m1)'+trace(invC1*Cov2+Cov1*invC2-2*eye(F));


    
            